options(echo=FALSE)

#This script simulates the numeric distribution of the test statistic for 100,000 values and estimates the 95% quantile for sample sizes n=30,40,...,100
#The output of this file is already containd in file "Table2.Weibull.R" line 200 (the cut-off points for the Weibull distribution example in Table 2).
#As such it is provided for reference purposes as most of the functions are already contained in file "Table2.Weibull.R" - the only difference is the last function
#which simply creates 100,000 test statistic values and applies the quantile() function to obtain the cut-off points. 


"Bandwidth"<-function(sample)
  #Rule of thumb bandwidth plug-in rules
  #input: sample data
  #output: a vector of two components:
  #Component 1 is the density bandwidth rule of Silverman (Density estimation for statistics and data analysis, Chapman and Hall, London, 1986), eq. 3.3.1
  #Component 2 is the distribution function bandwidth rule of Bowman, Halland Prvan (1988) (Bandwidth selection for the smoothing of distribution functions, Biometrika, Vol 85, page 801)
{
  n<-length(sample)
  StD<-sd(sample)
  IqR<-diff(quantile(sample, c(.25, .75)))
  tmp<-min(StD, IqR/1.34)
  c( 0.9*tmp*n^(-1/5) , 3.572* tmp * n^(-1/3)) #first entry is density simple plug in rule, second entry is distribution function simple plug in rule
}

"Epanechnikov"<-
  function(x){
    #Epanechikov kernel, square root of 5 is taken as 2.236068 (to save comp. time).
    ifelse(abs(x)< 2.236068, (3/4)*(1-((x^2)/5 ))/2.236068, ifelse(abs(x)>2.236068, 0,0))
  }


"IntEpanechnikov"<-
  function(x){ #Integrated Epanechikov kernel, square root of 5 is taken as 2.236068 (to save comp. time).
    ifelse(x< -2.236068, 0, ifelse(x> 2.236068, 1, .5- (2.236068*x*(x^2-15))/100))
  }


"HStar"<-function(xin, cens, xout)
  #Kaplan-Meier Estimate, slightly modified so that it avoids division by zero.
  #In: sample and censoring indicators, i.e. xin and cens correspond to $(Z_i, \Lambda_i)$ in manuscript 
  #(in fact the first two lines of the routine ensure that this will be the case anyway). The xout argument denotes the points where the estimate will be evaluated
  #the most common option is to use xin in the place of xout
  #Out: 1-\hat H^\star(X_i) (as given on the manuscript)
{
  ordxin <-  sort(xin) #order input data=Xi
  ordcens <- cens[order(xin)] #order censoring =Di
  n <- length(xin)
  nn <- length(xout)
  vKuse <- sapply(1:nn, function (i, ordxin, xout) max(c(max(which(ordxin <= xout[i]))
                                                         , min(which(xout[i] <= ordxin)))), ordxin, xout)
  vK <- vKuse - 1
  Huse<-sapply(vK, function (i,n,ordcens) ((n-1:i+1)/(n-1:i+2))^(1-ordcens[1:i]), n, ordcens)
  result<-sapply(Huse, prod)
  result[order(xin)]<-result
  result
}


"GPPlugInBand"<-function(xin, Di)
  #Gulati and Padgett (1996) (Families of smooth confidence bands for the survival function under the general random censorship model, Lifetime Data Anal., 2 349-362)
  #survival function, MSE optimal bandwidth selection rule (see equation 7 in the manuscript)
  #In: sample and censoring indicators
  #Out: vector of bandwidth values (ready to use in the \hat S_n estimate)
{
  n<-length(xin)
  Mu2sq<- 1 #specifically for Epanechnikov
  intxKW<-0.2874945 #specifically for Epanechnikov
  ThetaHat<-sum(xin)/sum(Di) #estimate theta for reference distribution
  arg1Use <- exp(-xin/ThetaHat ) #the body of the f,f' definition is evaluated once only to save comp. time
  FHat<- 1/ThetaHat * arg1Use #\tilde f
  FHatDer<- -1/ThetaHat^2 * arg1Use #\tilde f'
  KMUse<- HStar(xin , Di, xin ) #kaplan-meier
  
  ((2*FHat * intxKW )/ (n*KMUse * FHatDer^2 * Mu2sq))^(1/3) #apply the definition and return result 
}

"MPDensityPlugInBand"<-function(xin, Di)
  #Marron and Padgett (1986) (Asymptotically optimal bandwidth selection for kernel density estimators from randomly right censored samples, Ann. Statist, 15, 1520-1535)
  #density function, MSE optimal bandwidth selection rule (see formula for h_0 in the manuscript)
  #In: sample and censoring indicators
  #Out: vector of bandwidth values (ready to use in the f_n estimate)
{
  n<-length(xin)
  RK<-  (3/25) * sqrt(5) #specifically for Epanechnikov
  ThetaHat<-sum(xin)/sum(Di) #estimate theta for reference distribution
  arg1Use <- exp(-xin/ThetaHat )#the body of the f,f' definition is evaluated once only to save comp. time
  FHat<- 1/ThetaHat * arg1Use  #\tilde f
  FHatSecondDer<- 1/ThetaHat^3 * arg1Use #\tilde f'' 
  KMUse<- HStar(xin , Di, xin ) #kaplan-meier
  
  ((RK*FHat / KMUse )/ ( 4*( 1/2)^2 * n* FHatSecondDer^2 ))^(1/5)  #apply the definition and return result
}


"TestStatistic"<-function(xin, Di, xout,  kfun, Intkfun, H0dist, H1dist, p1,p2)
{
  #The test statistic #\tilde T_{n,\ast} / \sigma(\tilde T_{n,\ast})
  #in: sample (xin),  censoring indicators (Di), xout (set equal to xin), kfun (Epabechnikov kernel), Intkfun (Integrated Epanechnikov kernel)
  #    H0dist (Distribution under the null), H1dist (alternative distribution), p1,p2 (paramaters for the null distribution), p3,p4 (parameters under the alternative)
  #out: value of the test statistic
  TrueSurv<-switch(H0dist, gamma=1-pgamma(xin, shape=p1,scale=p1), weib=1-pweibull(xin, shape=p1,scale=p1), expon=1-pexp(xin, rate=p1))
  Rw<- (26/35) * sqrt(5) #specifically for Epanechnikov
  RK<- (3/25) * sqrt(5) #specifically for Epanechnikov
  
  k<-1/2   ## k =\mu_2^2(K)/2 #specifically for Epanechnikov
  s<-  10.36478   #this is the S functinal in the paper, specifically for Epanechnikov
  n<-length(xin)
  KaplanMeier <- HStar(xin , Di , xin ) # Kaplan-Meier
  arg1<-Di/KaplanMeier #D_i/KM - used in the \hat S_n, f_n definitions
  h<-GPPlugInBand(xin, Di) #GP MSE bandwidth for the survival function estimate
  xXiDif<-(matrix(xin , ncol=n, nrow=n, byrow=T) - xin)#for the argument of the kernel
  xXiDifUse<-(matrix(-xin , ncol=n, nrow=n, byrow=T) - xin) # to be used for reflection in the density estimate - Jones (1993)
  out <- colSums(Intkfun(xXiDif /h ) ) #calculate \sum_{i=1}^n W(x-X_i)/h
  hatSn <- 1-out/n  #calculate \hat S_n
  h<-MPDensityPlugInBand(xin, Di)*0.45 #f_n optimal bandwidth 
  outdesn <-  colSums(kfun(xXiDif/h) * arg1 + kfun(xXiDifUse/h) * arg1    ) #calculate \sum_{i=1}^n K(x-X_i)/h + K(-x-X_i)/h (i.e.with reflection)
  hatfn <-  outdesn/(n * h ) #f_n estimate
  h2<-  (4/(n*(1+2*1+2)))^(1/(1+2*1+4))*sd(xin) #derivative optimal bandwidth (simple plugin rule)
  hatfnder<- colSums(xXiDif/h2 * kfun(xXiDif/h2)  * arg1  +  xXiDifUse/h2 * kfun(xXiDifUse/h2)  * arg1) # apply equation 10 in manuscript (with reflection)
  hatfnder<- hatfnder/(h2^2 * n * 2*k) # 2*k because k=\mu_2^2/2 and s2 in Jones (1993) paper is s2=\mu_2^2. Also: the - sign from the estimate is removed because of S''_n(x) = -f'_n(x)
  # \tilde T_{n,\ast}:
  h<-Bandwidth(xin)[2] #get distribution function bandwidth for the statistic
  hatfnOverKaplanMeier <-hatfn/KaplanMeier
  hatsigma2 <- ( h*   Rw *  sum(hatfnOverKaplanMeier ) - h^2   * RK * sum( (1-hatSn )^2 ))/n^2 #calculate \sigma_2^2
  Tn<-  (sum( (hatSn  - TrueSurv)^2 )- h^2 *k * sum(hatfnder^2))/n - hatsigma2  #caluclate \tilde T_{n,\ast}
  #Standard deviation  \tilde T_{n,\ast}:
  s1sq<- sum(hatfnOverKaplanMeier^2) #calculate \hat \sigma_1
  s4sq<- sum(hatfnder^2 * hatfnOverKaplanMeier - hatfnder^2 *(1-hatSn ))#calculate \hat \sigma_4
  lamhat<- n*h^3
  stdevTn<- (s1sq *sqrt(2*lamhat) + 2* lamhat^(1/3) * s4sq * k )* (s/n) #apply equation 11 in manuscript
  
  TestStatisticVal<-  n^(3/2) * Tn/stdevTn #apply the d(n) factor
  TestStatisticVal #return the test statistic value
}

set.seed(157)
ItNumber<-100000


#SampleSize<-50

CriticalValues<-function(ItNumber, SampleSize, dist, p1, p2, level, censparam, Epanechnikov, IntEpanechnikov )
{
  out<-sapply(1:ItNumber, function(i, SampleSize, dist, p1, p2, censparam, Epanechnikov, IntEpanechnikov )
  {
    data<- switch(dist, weib=rweibull(SampleSize, p1,p2), gamma=rgamma(SampleSize, , shape=p1,scale=p1), expon=rexp(SampleSize))
    cens<-runif(SampleSize, max = censparam)#used for compatibility of the routine in case of censoring, it is not needed nor used
    cen <- rep(1, times=length(data))#all observations are uncesnored
    TestStatistic(data, cen, data, Epanechnikov, IntEpanechnikov, dist, p1,p2)
  }, SampleSize, dist, p1, p2,  censparam, Epanechnikov, IntEpanechnikov)
  Result<-sort(out)
  cat("5% conf. lev. critical value for ", dist, "(",p1,p2,")"," = ", Result[ItNumber*(1-level) + 1], " ", quantile(Result, 1-level), "\n")
}


CriticalValues(ItNumber, 30, "weib", 1, 1, 0.05, 0.5751518, Epanechnikov, IntEpanechnikov )
CriticalValues(ItNumber, 40, "weib", 1, 1, 0.05, 0.5751518, Epanechnikov, IntEpanechnikov )
CriticalValues(ItNumber, 50, "weib", 1, 1, 0.05, 0.5741517, Epanechnikov, IntEpanechnikov )
CriticalValues(ItNumber, 60, "weib", 1, 1, 0.05, 0.5741517, Epanechnikov, IntEpanechnikov )
CriticalValues(ItNumber, 70, "weib", 1, 1, 0.05, 0.5751518, Epanechnikov, IntEpanechnikov )
CriticalValues(ItNumber, 80, "weib", 1, 1, 0.05, 0.5751518, Epanechnikov, IntEpanechnikov )
CriticalValues(ItNumber, 90, "weib", 1, 1, 0.05, 0.5731516, Epanechnikov, IntEpanechnikov )
CriticalValues(ItNumber, 100, "weib", 1, 1, 0.05, 0.5761518, Epanechnikov, IntEpanechnikov )

